/*
 * The Apache Software License, Version 1.1
 *
 *
 * Copyright (c) 2001 The Apache Software Foundation.  All rights 
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer. 
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:  
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Axis" and "Apache Software Foundation" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written 
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    nor may "Apache" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 */

package fr.univvalenciennes.publi.WebServices.adminPubli.GestionNews.News;

import java.sql.SQLException;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
public class NewsService {
	private static Log log = LogFactory.getLog("axis.news");
	/**  
	 *  Cette mthode s'attend a avoir un Order, dans lequel
	 *  on trouve l'uidnews.
	 */
	public News OrderNews(Order order) throws Exception {
		News response = new News();

		try {
			// On rcupre l'uidnews donn par le client
			String uidNews = order.getUidNews();

			// Puis on recherche la news dans la base de donnes
			SqlNews sqlNews = new SqlNews();
			response = sqlNews.getNews(uidNews);

		} catch (SQLException e) {
			log.error("SQLException : " + e);
		} catch (ClassNotFoundException e) {
			log.error("ClassNotFoundException : " + e);
		}
		return response;
	}

	/**  
	 *  Pour enlever une news dans la base de donnes  partir
	 *  de l'uid de la news.
	 */
	public void deleteNews(Order order) throws Exception {

		try {
			String uidNews = order.getUidNews();

			SqlNews sqlNews = new SqlNews();

			sqlNews.deleteNews(uidNews);

		} catch (SQLException e) {
			log.error("SQLException : " + e);
		} catch (ClassNotFoundException e) {
			log.error("ClassNotFoundException : " + e);
		}
	}
	public void deleteNewsByCanal(String uidCanal) throws Exception
	{

	  try{
		SqlNews sqlNews = new SqlNews();

		sqlNews.deleteNewsByCanal(uidCanal);

	  }catch(SQLException e){
		  log.error("SQLException : "+e);
	  } catch(ClassNotFoundException e){
		  log.error("ClassNotFoundException : "+e);
	  }
	}

	public void unpublishNewsByCanal(String uidCanal) throws Exception
	{

	  try{
		SqlNews sqlNews = new SqlNews();

		sqlNews.unpublishNewsByCanaux(uidCanal);

	  }catch(SQLException e){
		  log.error("SQLException : "+e);
	  } catch(ClassNotFoundException e){
		  log.error("ClassNotFoundException : "+e);
	  }
	}

	/**  
	 *  Permet de crer une nouvelle news dans la base de donnes
	 *
	 *  Les paramtres sont la news et le canal o elle doit figurer
	 */
	public void PutNews(News news, String uidcat) throws Exception {

		try {

			SqlNews sqlNews = new SqlNews();

			sqlNews.putNews(news, uidcat);

		} catch (SQLException e) {
			log.error("News service : SQLException : " + e);
		} catch (ClassNotFoundException e) {
			log.error("ClassNotFoundException : " + e);
		}
	}

	/**  
	 *  Mise  jour d'une news
	 */
	public void ModifyNews(News news) throws Exception {

		try {

			SqlNews sqlNews = new SqlNews();

			sqlNews.modifyNews(news);

		} catch (SQLException e) {
			log.error("SQLException : " + e);
		} catch (ClassNotFoundException e) {
			log.error("ClassNotFoundException : " + e);
		}
	}

	/**
	 * Supprime de la table NEWDISPATCHNEWS toutes les entres 
	 * attaches  la catgorie passe en paramtre
	 * @param uidCat
	 * @throws Exception
	 */
	public void unpublishByCat(String uidCat) throws Exception {
		try {
			SqlNews sqlNews = new SqlNews();
			sqlNews.unpublishByCat(uidCat);
		} catch (SQLException e) {
			log.error("SQLException : " + e);
		} catch (ClassNotFoundException e) {
			log.error("ClassNotFoundException : " + e);
		}
	}

	/**
	 * Supprime de la table NEWS toutes les news attaches  
	 * la catgorie passe en paramtre dans la table NEWDISPATCHNEWS
	 * @param uidCat
	 * @throws Exception
	 */
	public void deleteNewsByCat(String uidCat) throws Exception {
		try {
			SqlNews sqlNews = new SqlNews();
			sqlNews.deleteNewsByCat(uidCat);
		} catch (SQLException e) {
			log.error("SQLException : " + e);
		} catch (ClassNotFoundException e) {
			log.error("ClassNotFoundException : " + e);
		}
	}
}
